@@ -8,13 +8,30 @@
 #"   \code{\link[grDevices]{recordPlot}}.
 plot_snapshot <- local({
   last_plot <- NULL
+  # help decide whether to keep plots when multiple plots on one screen
+  mfg_init <- NULL
+  mfg_changed <- FALSE
 
   function(incomplete = FALSE) {
     if (is.null(dev.list())) return(NULL)
 
-    pos <- par("mfg")[1:2]
-    size <- par("mfg")[3:4]
-    if (!incomplete && !identical(pos, size)) return(NULL)
+    mfg <- par("mfg")
+    if (identical(mfg, rep(1L, 4)) || incomplete) {
+      mfg_init <<- NULL
+      mfg_changed <<- FALSE
+    } else {
+      # now there is a multi-col/row layout
+      if (is.null(mfg_init)) {
+        mfg_init <<- mfg
+      } else {
+        if (identical(mfg_init, mfg)) {
+          if (!mfg_changed) return(NULL)
+        } else {
+          mfg_changed <<- TRUE
+          return(NULL)
+        }
+      }
+    }
 
     plot <- recordPlot()
     if (is_par_change(last_plot, plot) || identical(last_plot, plot)) {
@@ -35,7 +52,11 @@ is_par_change <- function(p1, p2) {
   n2 <- length(calls2)
 
   if (n2 <= n1) return(FALSE)
-  if (!identical(calls1, calls2[1:n1])) return(FALSE)
+  i1 <- seq_len(n1)
+  if (!identical(calls1, calls2[i1])) return(FALSE)
+  # also check if the content of the display list is still the same (note we
+  # need p1[[1]][] as well because [] turns a dotted pair list into a list)
+  if (!identical(p1[[1]][i1], p2[[1]][i1])) return(FALSE)
 
   last <- calls2[(n1 + 1):n2]
   all(last %in% empty_calls)
